#!/usr/bin/env python
# _*_ coding:utf-8 _*_

"""
 Copyright: 2016-2018 ROS小课堂 www.corvin.cn
 Author: corvin
 Description:
    A Python driver for the Arduino microcontroller
 History:
    20181024: initial this source code file.
"""

import os
import time
import sys, traceback
from serial.serialutil import SerialException
from serial import Serial


class Arduino:
    ''' Configuration Arduino Board Parameters
    '''
    def __init__(self, port="/dev/ttyACM0", baudrate=57600, timeout=0.5):
        self.port     = port
        self.baudrate = baudrate
        self.timeout  = timeout
        self.interCharTimeout = timeout/30.

    # connect arduino board
    def connect(self):
        try:
            self.port = Serial(port=self.port, baudrate=self.baudrate, timeout=self.timeout)
            # give the firmware time to wake up.
            time.sleep(1)
        except SerialException:
            print "Serial Exception:"
            print sys.exc_info()
            print "Traceback follows:"
            traceback.print_exc(file=sys.stdout)
            print "Cannot connect to Arduino!"
            os._exit(2)

    def close(self):
        ''' Close the serial port.
        '''
        self.port.close()

    def recv(self, timeout=0.5):
        timeout = min(timeout, self.timeout)
        ''' This command should not be used on its own: it is called by the execute commands
            below in a thread safe manner.  Note: we use read() instead of readline() since
            readline() tends to return garbage characters from the Arduino
        '''
        c = ''
        value = ''
        attempts = 0
        while c != '\r':
            c = self.port.read(1)
            value += c
            attempts += 1
            if attempts * self.interCharTimeout > timeout:
                return None

        value = value.strip('\r')
        return value

    def recv_int_array(self):
        try:
            values = self.recv(self.timeout).split()
            return map(int, values)
        except:
            return []

""" Basic test for connectivity """
if __name__ == "__main__":
    portName = "/dev/ttyACM0"
    baudRate = 57600

    myArduino = Arduino(port=portName, baudrate=baudRate, timeout=0.5)
    print "Ready to connect arduino serial port..."
    myArduino.connect()

    print "Ready to read data ..."
    time.sleep(1)
    Data = []
    Data = myArduino.recv_int_array()
    print Data

    myArduino.close()
    print "Now disconnect arduino serial port..."

